#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Res_Fileversion=1.34
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
;-------------------------------------------------------------------------------------------------------------------
; Project:  mame4to8  - v1.33    2/4/2023
; Member:  'Justin' from BYOAC forums
;
; Checks rom for 4/8 way and triggers commands on the TOS GRS motorized restrictor plate. (for SANWA joysticks)
; >> https://thunderstickstudio.com/collections/joystick/products/tos-grs-4-to-8-way-restrictor-all-in-one-kit
;
; This can be used in HyperSpin basic installations (even without HyperLaunch/RocketLaunch).
; It allows for HyperSpin to launch a MAME ROM while first checking if it uses 4 or 8 way joysticks,
; and then sends a command to the Thunderstick TOS GRS board via the tos428 utility to move the restrictor plates.
;
; Compile to .exe file using the AutoIt tool.
;
; Place your newly compiled mame4to8.exe file in the same folder as your mame.exe folder.
;
; Finally, modify your HyperSpin setup to use mame4to8.exe instead of the standard mame.exe as your
; 'emulator'. You can make that simple configuration change in the HyperHQ setup screens.
;
; It may be possible to use this with other frontends.
;
; In case you need to manually launch, usage format is:  mame4to8.exe romname -argument1 -argument2  (same as MAME)
;
; Project thread/link:  http://forum.arcadecontrols.com/index.php/topic,167355.msg1762688.html
;-------------------------------------------------------------------------------------------------------------------
#pragma compile(Icon, "mame4to8.ico")
#include <File.au3>

;Initialize a new ini file if it doesnt already exist (during first run of the app only, because file doesn't exist.)
;Define Default values for ini variables.
$inifile = "\mame4to8.ini"
$inifile = @ScriptDir & $inifile
$defMameName = '"mame.exe"'
$defPathTOS428cl = '"' & FindTOSexe() & '"'
$defCOMPort = '"AUTO"'
$defWhichRestrictor = '"ALL"'
$defExitState = '"8"'

If FileExists($inifile) = 0 Then      ;Write ini with default values
	IniWrite($inifile, "MAIN_PARAMS", "MameName", $defMameName)
	IniWrite($inifile, "MAIN_PARAMS", "PathTOS428cl", $defPathTOS428cl)
	IniWrite($inifile, "MAIN_PARAMS", "COMPort", $defCOMPort)
	IniWrite($inifile, "MAIN_PARAMS", "WhichRestrictor", $defWhichRestrictor)
	IniWrite($inifile, "MAIN_PARAMS", "ExitState", $defExitState)
	IniWrite($inifile, "EXCEPTIONS", "romname1", """8""")
	IniWrite($inifile, "EXCEPTIONS", "romname2", """4""")
	IniWrite($inifile, "EXCEPTIONS", "romname3", """4""")
	;Now add header comment section to file
	;Create data to be written to the file.
	_FileWriteToLine($inifile, 1, @CRLF)
	_FileWriteToLine($inifile, 2, "# Configure these parameters as follows:")
	_FileWriteToLine($inifile, 3, "#")
	_FileWriteToLine($inifile, 4, "# MameName")
	_FileWriteToLine($inifile, 5, "#     The name of your MAME executable.")
	_FileWriteToLine($inifile, 6, "#     In case you use a custom compile with a different name.")
	_FileWriteToLine($inifile, 7, "#")
	_FileWriteToLine($inifile, 8, "# PathTOS428cl")
	_FileWriteToLine($inifile, 9, "#     The path to the TOS428cl utility.")
	_FileWriteToLine($inifile, 10, "#")
	_FileWriteToLine($inifile, 11, "# COMPort ")
	_FileWriteToLine($inifile, 12, "#     The COM port associated to your TOS GRS board. ")
	_FileWriteToLine($inifile, 13, "#     COM6,  COM22  etc., or use AUTO for autodetection.")
	_FileWriteToLine($inifile, 14, "#")
	_FileWriteToLine($inifile, 15, "# WhichRestrictor ")
	_FileWriteToLine($inifile, 16, "#     Which restrictor to move.  Set to 1,2,3,4 or ALL. ")
	_FileWriteToLine($inifile, 17, "#     Note: Only option ALL works for now. Bug?")
	_FileWriteToLine($inifile, 18, "#")
	_FileWriteToLine($inifile, 19, "# ExitState")
	_FileWriteToLine($inifile, 20, "#     The position of restrictors at exit.  ")
	_FileWriteToLine($inifile, 21, "#     Set to 4, 8, K (keep last), or D (to use circuit board default).")
	_FileWriteToLine($inifile, 22, "#")
	_FileWriteToLine($inifile, 23, "# [EXCEPTIONS]")
	_FileWriteToLine($inifile, 24, "#     Use this section to set exceptions for a given game / ROM. Ex.:  romname = ""8""")
	_FileWriteToLine($inifile, 25, "#     Entirely optional, as mame4to8 app automatically detects the correct joystick 8/4 way position for each ROM.")
	MsgBox(0, "INITIALIZED", "***FIRST LAUNCH!***" & @CRLF & @CRLF & "INI file has been created." & @CRLF & "Please edit ""mame4to8.ini"" to meet your needs." _
			 & @CRLF & @CRLF & "Click OK to open INI file...")
	Run("notepad.exe " & $inifile, @WindowsDir, @SW_MAXIMIZE)
	Exit
EndIf

;If no argument (ROM name) passed, then exit.  Must have a ROM name to work with!
If $CmdLine[0] == "0" Then
	MsgBox(8192, "Error", "You must pass a ROM name onto this script")
	Exit
EndIf

;Now clean the ROM name in case it has a period character, as it often does with Hyperspin for some odd reason.
$ROMname = $CmdLine[1]
while StringInStr($ROMname,".")
	$ROMname = StringTrimRight($ROMname,1)
WEnd

;$ROMname = StringReplace($ROMname, ".zip", "")
;$ROMname = StringReplace($ROMname, ".", "")


;Start by reading  all parameters from .ini file.
;We will work with non-quoted entries.  If quotes are present we will strip them here.
;Note that $WhichRestrictor must be lowercase, so we will take care of this here as well.
If FileExists($inifile) = 1 Then
	$MameName = StringReplace(IniRead($inifile, "MAIN_PARAMS", "MameName", $defMameName), '"', '')
	$PathTOS428cl = StringReplace(IniRead($inifile, "MAIN_PARAMS", "PathTOS428cl", $defPathTOS428cl), '"', '')
	$COMPort = StringReplace(IniRead($inifile, "MAIN_PARAMS", "COMPort", $defCOMPort), '"', '')
	$WhichRestrictor = StringLower(StringReplace(IniRead($inifile, "MAIN_PARAMS", "WhichRestrictor", $defWhichRestrictor), '"', ''))
	$ExitState = StringReplace(IniRead($inifile, "MAIN_PARAMS", "ExitState", $defExitState), '"', '')
	$ExceptionWay = StringReplace(IniRead($inifile, "EXCEPTIONS", $ROMname, ""), '"', '')
Else
	MsgBox(0, "Error", "No .ini file found!")
EndIf

; ---------Define here a few useful functions-------------------------------------------------------
;Function to help locate where TOS428 is installed...
Func FindTOSexe()
	Local $TOSexeDir
	If FileExists(@HomeDrive & "\" & "Program Files\tos428cl\TOS428cs.exe") Then
		$TOSexeDir = @HomeDrive & "\" & "Program Files\tos428cl\"
	ElseIf FileExists(@HomeDrive & "\" & "Program Files (x86)\tos428cl\TOS428cs.exe") Then
		$TOSexeDir = @HomeDrive & "\" & "Program Files (x86)\tos428cl\"
	ElseIf FileExists("D:\Program Files\tos428cl\TOS428cs.exe") Then
		$TOSexeDir = "D:\Program Files\tos428cl\"
	ElseIf FileExists("D:\Program Files (x86)\tos428cl\TOS428cs.exe") Then
		$TOSexeDir = "D:\Program Files (x86)\tos428cl\"
	Else
		$TOSexeDir = "C:\Program Files\tos428cl\"   ;assume this general install location.
	EndIf
	Return $TOSexeDir
EndFunc   ;==>FindTOSexe

;Define function to change the TOS GRS 4/8 way.
Func TOSsetway($TOSrestrictor, $TOSway)
	Run(@ComSpec & ' /c ' & """" & $PathTOS428cl & "tos428cs"" " & $COMPort & " setway," & $TOSrestrictor & "," & $TOSway & "", $PathTOS428cl, @SW_HIDE)
EndFunc   ;==>TOSsetway

;Define function to get simple params from TOS428.
Func GetParamTOS428($TOScmd)
	Local $iPID = Run($PathTOS428cl & "\tos428cl.exe " & $COMPort & " " & $TOScmd, $PathTOS428cl, @SW_HIDE, 0x6)
	Local $TOS428Msg
	While 1
		$TOS428Msg &= StdoutRead($iPID)
		If @error Then ExitLoop
		Sleep(10)
	WEnd
	$TOS428Msg = StringReplace($TOS428Msg, @CRLF, "") ;remove nasty line feed causing issues.
	Return $TOS428Msg
EndFunc   ;==>GetParamTOS428
; ---------------------------------------------------------------------------------------------------


;Now we identify and set up the TOS GRS port.
$PortFound = 0 ;assume no port found at start
If $COMPort = "AUTO" Then  ;auto scan for port.
	Local $iPID = Run($PathTOS428cl & "\tos428cl.exe getport", $PathTOS428cl, @SW_HIDE, 0x6)
	Local $COMportMsg
	While 1
		$COMportMsg &= StdoutRead($iPID)
		If @error Then ExitLoop
		Sleep(10)
	WEnd
	$COMportMsg = StringReplace($COMportMsg, @CRLF, "") ;remove nasty line feed causing issues.
	If StringInStr($COMportMsg, "tos428 found") <> 0 Then
		$PortFound = 1
		$COMPort = "COM" & StringRight($COMportMsg, StringLen($COMportMsg) - StringInStr($COMportMsg, "COM") - 2)
	ElseIf StringInStr($COMportMsg, "Port not found") <> 0 Then
		MsgBox(0, "Error", "Port not found with Auto setting! Try configuring port manually in script.")
	Else
		MsgBox(0, "Error", "Error Auto detecting port.")
	EndIf
Else  ;else we just use the COMport as defined in the INI.
	If StringInStr($COMPort, "COM") = 0 Then
		$COMPort = "COM" & $COMPort ;in case the user forgot to define as COMXX format.
	Else
		;leave as is but ensure uppercase 'COM' in the ini file or else fails.
		$COMPort = StringUpper($COMPort)
	EndIf
	$PortFound = 1
EndIf



;Now figure out what WAY to set game to...
If $PortFound = 1 Then
	If $ExceptionWay <> "" Then ;if there is an exception set up for a given ROM in the INI file, then use that setting.
		TOSsetway($WhichRestrictor, $ExceptionWay)
	Else ;else use MAME to generate an XML informational file for the ROM being launched, and save the file locally.
		RunWait(@ComSpec & " /c " & $MameName & " -lx " & $ROMname & " > mame4to8info.xml", @ScriptDir, @SW_HIDE)
		;Now let's read the file and find out if the game uses a 4 way or 8 way joystick, and trigger the TOS GRS accordingly.
		$file = FileOpen("mame4to8info.xml", 0)
		$read = FileRead($file)
		If $read <> "" And @error <> -1 Then
			;scan ROM file to see if it needs a 4 way joystick (4 way, 2 way, 2 way vertical, etc).
			If StringInStr($read, "ways=""4""") Or StringInStr($read, "ways=""2""") Or StringInStr($read, "ways=""vertical2""") Then
				TOSsetway($WhichRestrictor, 4)
			Else ;no match, must need 8 way joystick then...
				TOSsetway($WhichRestrictor, 8)
			EndIf
		EndIf
		FileClose($file)
	EndIf
EndIf



;Now we run MAME passing the ROM name and any switches/arguments originally passed on int the original command...
;Assemble all arguments.
$arguments = ""
For $i = 1 To $CmdLine[0]
	$arguments = $arguments & " " & $CmdLine[$i]    ;gather and assemble all original passed on arguments.
Next

RunWait($MameName & " " & $arguments, @ScriptDir, @SW_HIDE)



;Finally, upon exiting MAME, set restrictors back to 4 or 8 way, or keep without change, or use circuit default as defined in INI file.
If $PortFound = 1 Then
	Select
		Case $ExitState = "4" Or $ExitState = "8"
			TOSsetway($WhichRestrictor, $ExitState)   ;use whatever way defined in INI (4 or 8).
		Case $ExitState = "d"    ;use circuit board default.
			TOSsetway($WhichRestrictor, GetParamTOS428("getstartupway"))
		Case $ExitState = "k"
			;do nothing (keep as is).
	EndSelect
EndIf

Exit
